import enum


class TagAttrs(enum.Enum):
    left_indentation = 'leftIndentation'
    is_italic = 'idItalic'
    is_bold = 'isBold'
    has_top_border = 'hasTopBorder'
    has_bottom_border = 'hasBottomBorder'
